#include <bits/stdc++.h>

using namespace std;
using i64 = long long;
using Mat = vector<vector<i64>>;
const i64 INF = 1e18;

Mat Step(const Mat& S) {
  const int di[] = {0, 1, 0, -1};
  const int dj[] = {1, 0, -1, 0};
  int n = S.size();

  Mat NS = S;

  i64 maxx = 0;
  for (int i = 0; i < n; ++i) 
    for (int j = 0; j < n; ++j) 
      maxx = max(maxx, S[i][j] / 2);

  for (int i = 0; i < n; ++i) {
    for (int j = 0; j < n; ++j) {
      if (S[i][j] / 2 == maxx) {
        for (int d = 0; d < 4; ++d) {
          int ni = i + di[d], nj = j + dj[d];
          if (ni >= 0 && nj >= 0 && ni < n && nj < n) 
            NS[i][j] -= 2, NS[ni][nj] += 2, NS[i][j] |= 1;
        }
      }
    }
  }
  return NS;
}

Mat SimSlow(Mat S, i64 k) {
  int n = S.size();
  for (i64 t = 0; t < k; ++t) 
    S = Step(S);
  return S;
}

i64 Examine(Mat S) {
  int n = S.size();
  i64 dmax = 0;
  for (i64 t = 0; t < 100000; ++t) {
    Mat NS = SimSlow(S, 1);
    i64 maxx = -INF, minn = INF;
    for (int i = 0; i < n; ++i)
      for (int j = 0; j < n; ++j) {
        if (NS[i][j] & 1) {
          maxx = max(maxx, NS[i][j] / 2);
          minn = min(minn, NS[i][j] / 2);
        }
      }
    dmax = max(dmax, maxx - minn);
    S = NS;
  }
  return dmax;
}

pair<int, int> FindPeriod(Mat S) {
  int n = S.size();

  auto norm = [&](Mat S) {
    i64 maxx = 0;
    for (int i = 0; i < n; ++i) {
      for (int j = 0; j < n; ++j) {
        if (S[i][j] % 2 == 1)
          maxx = max(maxx, S[i][j] / 2);
      }
    }
    for (int i = 0; i < n; ++i) {
      for (int j = 0; j < n; ++j) {
        if (S[i][j] % 2 == 1)
          S[i][j] = INF - 2 * maxx + S[i][j];
        else 
          S[i][j] = 0;
      }
    }
    return S;
  };

  int tail = 0, period = 0;
  auto S1 = norm(S), S2 = norm(S);
  do { S1 = norm(Step(S1)); S2 = norm(Step(Step(S2))); ++tail; } while (S1 != S2);
  do { S1 = norm(Step(S1)); S2 = norm(Step(Step(S2))); ++period; } while (S1 != S2);
  return {tail, period};
}

Mat SimFast(Mat S, i64 k) {
  int n = S.size();
  S = SimSlow(S, 1), k--;
  const int slack = 6;

  int ocnt = -1;

  while (k > 0) {
    int ncnt = 0;
    for (int i = 0; i < n; ++i)
      for (int j = 0; j < n; ++j)
        ncnt += (S[i][j] % 2);
    assert(ncnt > ocnt);
    ocnt = ncnt;

    auto [tail, period] = FindPeriod(S);
    cerr << "tail=" << tail << " period=" << period << endl;
    assert(tail == period);
    if (tail + slack * period >= k) 
      return SimSlow(S, k);
    
    auto S1 = SimSlow(S, tail);
    auto S2 = SimSlow(S1, period);

    if (S1 == S2) 
      return SimSlow(S1, (k - tail) % period);

    bool bad = false;
    for (int i = 0; i < n; ++i)
      for (int j = 0; j < n; ++j)
        if (S[i][j] % 2 == 0 && S2[i][j] % 2 == 1)
          bad = true;
    
    if (bad) {
      S = SimSlow(S, tail + slack * period);
      k -= tail + slack * period;
      continue;
    }

    i64 maxx = 0, loss = -1;
    for (int i = 0; i < n; ++i) {
      for (int j = 0; j < n; ++j) {
        if (S[i][j] % 2) {
          maxx = max(maxx, S1[i][j]);
          if (loss == -1) loss = S1[i][j] - S2[i][j];
          else assert(loss == S1[i][j] - S2[i][j]);
        } 
      }
    }
    // cerr << "loss=" << loss << endl;
    assert(loss > 0);

    i64 max_iters = (k - tail) / period;
    for (int i = 0; i < n; ++i) {
      for (int j = 0; j < n; ++j) {
        if (S[i][j] % 2 == 0) {
          // S1[i][j] + iter * (S2[i][j] - S1[i][j]) == maxx - iter * loss
          // iter * (S2[i][j] - S1[i][j] + loss) == maxx - S1[i][j]
          max_iters = min(max_iters, (maxx - S1[i][j]) / (S2[i][j] - S1[i][j] + loss));
        }
      }
    }
    max_iters = max(0LL, max_iters - slack);
    // cerr << "max_iters=" << max_iters << endl;

    for (int i = 0; i < n; ++i) {
      for (int j = 0; j < n; ++j) {
        S[i][j] = S1[i][j] + (S2[i][j] - S1[i][j]) * max_iters;
      }
    }
    k = k - tail - max_iters * period;
    assert(k >= slack * period);
    i64 extra = min(k, 1LL * (slack + 2) * period);
    S = SimSlow(S, extra);
    k -= extra;
  }
  return S;
}

void Test() {
  mt19937_64 rng(42);
  i64 maxd = 0;
  i64 M = 100;
  for (int it = 0; ; ++it) {
    int n = 3;
    Mat S(n, vector<i64>(n));
    for (int i = 0; i < n; ++i) {
      for (int j = 0; j < n; ++j) {
        S[i][j] = 2 * ((rng() % (M - M/10)) + M/10);
      }
    }
    i64 k = rng() % INF + 1;
    cerr << k << endl;
    for (int i = 0; i < n; ++i) {
      for (int j = 0; j < n; ++j) {
        cerr << S[i][j] / 2 << " ";
      }
      cerr << endl;
    }

    auto SF = SimFast(S, k);
    auto SS = SF; // SimSlow(S, k);
    if (SF != SS) {
      cout << endl;
      

      for (int i = 0; i < n; ++i) {
        cout << "   ";
        for (int j = 0; j < n; ++j) {
          cout << SF[i][j] / 2 << " ";
        }
        cout << "   ";
        for (int j = 0; j < n; ++j) {
          cout << SS[i][j] / 2 << " ";
        }
        cout << endl;
      }
      assert(false);
    }
    // maxd = max(maxd, Examine(S));
    cerr << "it=" << it << " OK\n";
  }
}

int main() {
  
  int T; cin >> T;
  while (T--) {
    int n = 3;
    i64 k; cin >> k;
    Mat S(n, vector<i64>(n));
    for (int i = 0; i < n; ++i)
      for (int j = 0; j < n; ++j) {
        cin >> S[i][j];
        S[i][j] *= 2;
      }
    S = SimFast(S, k);
    for (int i = 0; i < n; ++i) {
      for (int j = 0; j < n; ++j)
        cout << S[i][j] / 2 << " ";
      cout << '\n';
    }
  }

  return 0;
}